package com.icing.dialer.services

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.os.Build
import android.telecom.Call
import android.telecom.InCallService
import android.util.Log
import androidx.core.app.NotificationCompat
import com.icing.dialer.activities.MainActivity
import io.flutter.plugin.common.MethodChannel

class MyInCallService : InCallService() {
    companion object {
        var channel: MethodChannel? = null
        var currentCall: Call? = null
        private const val TAG = "MyInCallService"
        private const val NOTIFICATION_CHANNEL_ID = "incoming_call_channel"
        private const val NOTIFICATION_ID = 1
    }

    private val callCallback = object : Call.Callback() {
        override fun onStateChanged(call: Call, state: Int) {
            super.onStateChanged(call, state)
            val stateStr = when (state) {
                Call.STATE_DIALING -> "dialing"
                Call.STATE_ACTIVE -> "active"
                Call.STATE_DISCONNECTED -> "disconnected"
                Call.STATE_DISCONNECTING -> "disconnecting"
                Call.STATE_RINGING -> "ringing"
                else -> "unknown"
            }
            Log.d(TAG, "State changed: $stateStr for call ${call.details.handle}")
            channel?.invokeMethod("callStateChanged", mapOf(
                "callId" to call.details.handle.toString(),
                "state" to stateStr
            ))
            if (state == Call.STATE_RINGING) {
                showIncomingCallNotification(call.details.handle.toString().replace("tel:", ""))
            } else if (state == Call.STATE_DISCONNECTED || state == Call.STATE_DISCONNECTING) {
                Log.d(TAG, "Call ended: ${call.details.handle}")
                channel?.invokeMethod("callEnded", mapOf("callId" to call.details.handle.toString()))
                currentCall = null
                cancelNotification()
            }
        }
    }

    override fun onCallAdded(call: Call) {
        super.onCallAdded(call)
        currentCall = call
        val stateStr = when (call.state) {
            Call.STATE_DIALING -> "dialing"
            Call.STATE_ACTIVE -> "active"
            Call.STATE_RINGING -> "ringing"
            else -> "dialing"
        }
        Log.d(TAG, "Call added: ${call.details.handle}, state: $stateStr")
        channel?.invokeMethod("callAdded", mapOf(
            "callId" to call.details.handle.toString(),
            "state" to stateStr
        ))
        if (stateStr == "ringing") {
            showIncomingCallNotification(call.details.handle.toString().replace("tel:", ""))
        }
        call.registerCallback(callCallback)
    }

    override fun onCallRemoved(call: Call) {
        super.onCallRemoved(call)
        Log.d(TAG, "Call removed: ${call.details.handle}")
        call.unregisterCallback(callCallback)
        channel?.invokeMethod("callRemoved", mapOf("callId" to call.details.handle.toString()))
        currentCall = null
        cancelNotification()
    }

    override fun onCallAudioStateChanged(state: android.telecom.CallAudioState) {
        super.onCallAudioStateChanged(state)
        Log.d(TAG, "Audio state changed: route=${state.route}")
        channel?.invokeMethod("audioStateChanged", mapOf("route" to state.route))
    }

    private fun showIncomingCallNotification(phoneNumber: String) {
        val notificationManager = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager

        // Create notification channel (Android 8.0+)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val channel = NotificationChannel(
                NOTIFICATION_CHANNEL_ID,
                "Incoming Calls",
                NotificationManager.IMPORTANCE_HIGH
            ).apply {
                description = "Notifications for incoming calls"
                enableVibration(true)
                setShowBadge(true)
            }
            notificationManager.createNotificationChannel(channel)
        }

        // Intent to open MainActivity with phone number
        val intent = Intent(this, MainActivity::class.java).apply {
            flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
            putExtra("phoneNumber", phoneNumber)
            putExtra("isIncomingCall", true)
        }
        val pendingIntent = PendingIntent.getActivity(
            this,
            0,
            intent,
            PendingIntent.FLAG_UPDATE_CURRENT or (if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) PendingIntent.FLAG_IMMUTABLE else 0)
        )

        // Build notification
        val notification = NotificationCompat.Builder(this, NOTIFICATION_CHANNEL_ID)
            .setSmallIcon(android.R.drawable.ic_dialog_alert) // Replace with your app icon
            .setContentTitle("Incoming Call")
            .setContentText("Call from $phoneNumber")
            .setPriority(NotificationCompat.PRIORITY_HIGH)
            .setContentIntent(pendingIntent)
            .setAutoCancel(true)
            .setOngoing(true) // Keep visible until call ends
            .build()

        notificationManager.notify(NOTIFICATION_ID, notification)
        Log.d(TAG, "Notification shown for incoming call from $phoneNumber")
    }

    private fun cancelNotification() {
        val notificationManager = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        notificationManager.cancel(NOTIFICATION_ID)
        Log.d(TAG, "Notification canceled")
    }
}